Meteor.methods({

	/*
		Run once at startup:
			Loop through fetched friend list from Facebook to 
			check if friend list stored in Mongo is up to date.
			Remove friends that are no longer... friends... :-)
	*/
	
	updateFriends(friendArray) {
    	if (! Meteor.userId()) {
      		throw new Meteor.Error("not-authorized");
    	}
		var length = friendArray.length;

		for (var i=0; i<length; i++) {
			Friends.upsert({
			    id: friendArray[i].id,
			    owner: Meteor.userId()
			}, {
			    $set: {
			    	createdAt: new Date(),
        			name: friendArray[i].name,
			    }
			});

			/* // Made before I was aware of the upsert option.
			if (friend != undefined) {
				// ...updating possible name changes for users
				if (friend.name != friendArray[i].name) {
				Friends.update({
					id: friend.id,
					owner: Meteor.userId()
					},
						{
							$set: {
								name: friendArray[i].name
							}
						}
					);
	   				console.log("Friend updated");
				}
			} else {
				Friends.insert({
					name: friendArray[i].name,
					id: friendArray[i].id,
			    	createdAt: new Date(),
			    	owner: Meteor.userId()
				});
				console.log("Friend inserted");
			}
			*/
		}

		// Deletion of former friends.
		Friends.find({owner: Meteor.userId()}).forEach(function(user){
			var activeFriend = false;
			for (var i=0; i<length; i++) {
				if (friendArray[i].id==user.id) {
					activeFriend = true;
					console.log("Found friend");
					break;
				}
			}
			if (!activeFriend) {
				console.log("Deleting old friend");
				Friends.remove({
					id: user.id,
			    	owner: Meteor.userId()
				});
			}
		});
	},


	/*
		Add players to current game
		Input: Array containing players' Facebook IDs, and game ID.
	*/

	addFriendsToGame(players, gameId) {
    	if (! Meteor.userId()) {
      		throw new Meteor.Error("not-authorized");
    	}

	    Games.update({
	    	gameId: gameId,
		    owner: Meteor.userId()
	    },
    		{
    			$set: {
    				players: players
    			}
    		}
	    );

	    console.log('adding players ' + players + ' to gameId ' + gameId);

	},


	/*
		Find players already added to a game
	*/

	findPlayersInGame(gameId) {
		var game = Games.findOne({    	
			gameId: gameId,
		    owner: Meteor.userId()
		});

		if (game != undefined) {
			console.log("gameid found in find friends " + game.gameId + " name " + game.name + " players " + game.players);
			return game.players;
		} else {
			return [];
		}
	},


	findGameMasterGames(playerId) {
		return Games.find({ owner: Meteor.userId() });
	},


	/*
		Find games for a given player
	*/

	findPlayerGames(playerId) {
		console.log("playerid " + playerId);
		return Games.find({ players: playerId });

/*
		var games = Games.find({    	
		    players: playerId
		});
		var gameArray = [];
		games.forEach(function(game){
			console.log("GID " + game.gameId);
			gameArray.push(game.gameId);
		});
		return games;

*/
	},



	/*
		Add new game made by current user
	*/

	addGame(gameName) {
    	if (! Meteor.userId()) {
      		throw new Meteor.Error("not-authorized");
    	}

    	gameId = Random.id();

    	var gameIdTmp = Games.insert({
		    name: gameName,
		    gameId: gameId,
		    players: [],
		    createdAt: new Date(),
		    owner: Meteor.userId()
	    });

    	console.log("Add Game " + gameName + " gid " + gameId + " gidtmp " + gameIdTmp);

		var game = Games.findOne({    	
			gameId: gameId,
		    owner: Meteor.userId()
		});

		console.log(game);
		if (game != undefined) {
			console.log("gameid found in add game " + game.gameId + " name " + game.name);
		}

    	return gameId;
	},

	/*
		Delete game made by current user
	*/

	deleteGame(gameId) {
    	if (! Meteor.userId()) {
      		throw new Meteor.Error("not-authorized");
    	}

    	Games.remove({
		    gameId: gameId,
		    owner: Meteor.userId()
    	});
	},


	/*
		Delete question made by current user
	*/

	deleteQuestion(questionId) {
    	if (! Meteor.userId()) {
      		throw new Meteor.Error("not-authorized");
    	}


    	console.log("removing " + questionId);
    	// NOTE: Must also remove AnswerQuestion
    	Questions.remove({
		    questionId: questionId,
		    owner: Meteor.userId()
    	});

    	AnswerQuestions.remove({
		    questionId: questionId,
		    owner: Meteor.userId()
    	});


	},


	/*
		Add new question, answer, points and response for a game
	*/

  	addQuestion(gameId, questionId, questionValue, answerValue, congratulation, reward, points) {
      	if (! Meteor.userId()) {
        	throw new Meteor.Error("not-authorized");
      	}

      	// TEMPORARY HACK
      	if (questionId == null) {
      		return;
      	}

      	console.log(gameId + " " + questionValue + " " + answerValue + " " + congratulation + " " + reward + " " + points);

//      	questionId = Random.id();



		/* 	createdAt-date should not be changed when upserting.
			Check for old date before possibly setting new date.
		*/

		var existingQuestion = Questions.findOne({questionId: questionId, owner: Meteor.userId()});
		var date;

		if (existingQuestion == undefined) {
			date = new Date(); 
		} else {
			date = existingQuestion.createdAt;
		}

		Questions.upsert({
			    questionId: questionId,
			    owner: Meteor.userId(),
			    createdAt: date
			}, {
			    $set: {
			    	question: questionValue,
				    gameId: gameId,
			    	answer: answerValue,
			    	congratulation: congratulation,
			    	reward: reward,
			    	points: points,
			    	modifiedAt: new Date()
			    }
			});

		/*
			IMPORTANT: 	Change this to split up question in Question and AnswerQuestions collections!
						The reason for this is to not let the user get direct access to the answers,
						while at the same time have access for gamemaster to edit questions.
		*/


		/*
	    Questions.insert({
		    question: questionValue,
		    questionId: questionId,
		    gameId: gameId,
	    	answer: answerValue,
	    	congratulation: congratulation,
	    	reward: reward,
	    	points: points,
		    createdAt: new Date(),
		    owner: Meteor.userId()
	    });
	    */

 // ADD THIS at a later stage to make application more secure.
 /*
	    AnswerQuestions.upsert({
	    	questionId: questionId,
	    	answer: answerValue,
	    	congratulation: congratulation,
	    	reward: reward,
	    	points: points,
		    createdAt: new Date(),
		    owner: Meteor.userId()
	    });
	    //*/
	},

	editQuestion(questionId) {
		return AnswerQuestions.findOne({questionId: questionId, owner: Meteor.userId()});
	},

	
	/*
		Check for correct answer, and increase number of tries
	*/	

	checkCorrectAnswer(questionId, answerValue, facebookId) {
    	if (! Meteor.userId()) {
      		throw new Meteor.Error("not-authorized");
    	}

    	console.log("checkCorrectAnswer " + questionId + " for user " + Meteor.userId() + " with answerValue " + answerValue);

   		var userAnswer = UserQuestions.findOne({
   			questionId: questionId,
   			player: Meteor.userId()
   		});

   		var points = 0;
		var tries = 1;
   		if (userAnswer != undefined) {
   			if (userAnswer.completed) {
   				console.log("Returning true in checkCorrectAnswer");
   				return true;
   			}
	   		tries = userAnswer.tries;
	   		if (tries >= 1) {
		   		tries++;
	   		} else {
	   			tries = 1;
	   		}
	   		console.log("Tries " + tries);
   		} else {
	   		userAnswer = UserQuestions.insert({
   				questionId: questionId,
   				player: Meteor.userId(),
   				owner: Meteor.userId(),
   				facebookId: facebookId,
   				tries: tries,
   				points: points,
   				completed: false
   			});
   		}


//Change this
//    	var answer = AnswerQuestions.findOne({
    	var answer = Questions.findOne({
    		questionId: questionId
    	});

    	var completed = false;

    	if ((answer != undefined) && (answerValue == answer.answer)) {
    		console.log("WHOOP WHOOP !!!");
    		points = answer.points;
    		completed = true;
    	} else {
    		console.log("WHEEP WHEEP !!!");
    		completed = false;
    	}

    	UserQuestions.update({
   			questionId: questionId,
			owner: Meteor.userId()
		},
    		{
    			$set: {
    				player: Meteor.userId(),
    				tries: tries,
    				points: points,
    				completed: completed
    			}
    		}
		);

    	console.log("Returning " + completed);

		return completed;

	},


	/*
		Check if user already has completed a question
	*/

	checkPlayerCompletedQuestion(questionId) {
    	if (!Meteor.userId()) {
      		throw new Meteor.Error("not-authorized");
    	}
    	console.log("checkPlayerCompletedQuestion " + questionId + " for user " + Meteor.userId());

    	var userAnswer = UserQuestions.findOne({
    		questionId: questionId,
    		//Check if this is better
    		//player: Meteor.userId()
    		facebookId: Session.get('userFacebookId')
    	});

    	if (userAnswer != undefined) {
			return userAnswer.completed;
    	}

    	return false;
	},


	checkNumberOfCompletedQuestions(gameId, player) {
		var questions = Questions.find({gameId: gameId});
		var correctCounter = 0;

		questions.forEach(function(question){
			var userAnswer = UserQuestions.findOne({
	    		questionId: question.questionId,
    			player: player
    		});

	    	if (userAnswer != undefined) {
				if (userAnswer.completed) {
					correctCounter++;
				}
    		}
		});

   		correctCounter++;
		return correctCounter;
	},

	checkNumberOfQuestions(gameId) {
		var questions = Questions.find({gameId: gameId}).fetch();
		return questions.length;
	},

	getUserGame(args) {
		var gameUser = GameUsers.findOne({gameId: args[0], facebookId: args[1]});
		console.log(gameUser);
		return gameUser;
	},

	insertGameUser(gameId, facebookId) {
		GameUsers.insert({gameId: gameId, facebookId: facebookId, player: Meteor.userId(), owner: Meteor.userId()});
	},

	/*
		Get correct answer for a particular question
	*/

	getAnswerObject(questionId) {
    	if (!Meteor.userId()) {
      		throw new Meteor.Error("not-authorized");
    	}
    	var userAnswer = UserQuestions.findOne({
    		questionId: questionId,
    		player: Meteor.userId()
    	});

    	if (userAnswer != undefined && userAnswer.completed) {
//    		return AnswerQuestions.findOne({questionId: questionId});
    		return Questions.findOne({questionId: questionId});
		}
    	return false;
	},


	getTotalPoints(gameId, userId) {
    	if (!Meteor.userId()) {
      		throw new Meteor.Error("not-authorized");
    	}
  	 	
		var questions = Questions.find({gameId: gameId});
		var points = 0;

		questions.forEach(function(question){
			var userAnswer = UserQuestions.findOne({
	    		questionId: question.questionId,
    			player: userId
    		});

	    	if (userAnswer != undefined) {
				if (userAnswer.completed) {
					points += parseInt(userAnswer.points);
				}
    		}
		});

		if (points > 0) {
			return "Points achieved: " + points;
		}

		return;
	},



});

